#!/usr/bin/env python
# encoding: utf-8
import tornado.web
import json
import traceback
import logging

log = logging.getLogger("handlers.json")

class JSONRequest(tornado.web.RequestHandler):
    INDENT = None
    ENCODING = 'utf-8'
    ENSURE_ASCII = False
    SORT_KEYS = False
    __json = None

    def options(self, *args, **kwargs):
        self.finish()

    def prepare(self):
        self.clear_header('Content-Type')
        self.set_header('Access-Control-Allow-Methods', ", ".join(self.SUPPORTED_METHODS))
        self.set_header('Access-Control-Allow-Headers', "accept, origin, content-type, cookie")
        self.set_header('Access-Control-Allow-Credentials', 'true')
        self.set_header('Access-Control-Max-Age', 3600)

        if self.request.method == 'OPTIONS':
            return self.finish()

        self.content_type = 'application/json'
        self.set_header("Content-Type", "application/json; charset=%s" % self.ENCODING.lower())

    @property
    def json(self):
        if self.__json:
            return self.__json
        else:
            if self.content_type in self.request.headers.get('Content-Type', ''):
                try:
                    self.__json = json.loads(self.request.body) if self.request.body else {}
                    return self.__json
                except Exception as e:
                    log.debug(traceback.format_exc())
                    log.error(repr(e))
                    self.write_error(400)

    @staticmethod
    def default(obj):
        if hasattr(obj, '__dict__'):
            return obj.__dict__
        elif hasattr(obj, '_to_json'):
            return obj._to_json()
        else:
            return str(obj)

    def _jsonify(self, data):
        default = self.default
        return json.dumps(
            data,
            indent=self.INDENT,
            sort_keys=self.SORT_KEYS,
            ensure_ascii=self.ENSURE_ASCII,
            encoding=self.ENCODING,
            default=default
        )

    def response(self, data, finish=False):
        if not self._finished:
            self.write(self._jsonify(data))
            if finish:
                self.finish()


#!/usr/bin/env python
# encoding: utf-8
import os
import tornado.ioloop
from tornado.web import Application
from tornado.httpserver import HTTPServer
from tornado.log import app_log as log
from lumper.server import HANDLERS
from crew.master.tornado import Client
from pika import PlainCredentials


def run(args):
    crew_client = Client(
        host=args.rmq_address, port=args.rmq_port, virtualhost=args.rmq_vhost,
        credentials=PlainCredentials(username=args.rmq_user, password=args.rmq_password)
    )

    app = Application(
        args=args,
        handlers=HANDLERS,
        xsrf_cookies=False,
        cookie_secret=args.cookie_secret,
        debug=args.debug,
        reload=args.debug,
        gzip=args.gzip,
        crew=crew_client,
        timeout=args.timeout
    )

    http_server = HTTPServer(app, xheaders=True)
    http_server.listen(args.port, address=args.address)
    log.info('Server started {host}:{port}'.format(host=args.address, port=args.port))

    try:
        tornado.ioloop.IOLoop.instance().start()
    except Exception as e:
        log.exception(e)
        log.fatal("Server aborted by error: %r", e)

    return 0



#!/usr/bin/env python
# encoding: utf-8

import logging
import json
import os
import shutil
import traceback
import git
import re
import requests

from crew.worker import context, HandlerClass
from uuid import uuid4
from tempfile import gettempdir

log = logging.getLogger("builder")


class TemporaryFolder(object):

    def __init__(self):
        self._dir = os.path.join(gettempdir(), str(uuid4()))
        self._curdir = os.path.abspath(os.getcwd())

    def __enter__(self):
        assert not os.path.exists(self._dir)

        log.debug('Making directory: "%s"', self._dir)
        os.makedirs(self._dir)

        log.debug('Changing directory: "%s"', self._dir)
        os.chdir(self._dir)
        return self._dir

    def __exit__(self, exc_type, exc_val, exc_tb):
        log.debug('Changing directory: "%s"', self._curdir)
        os.chdir(self._curdir)

        if os.path.exists(self._dir):
            log.debug('Deleting directory: "%s"', self._dir)
            shutil.rmtree(self._dir)


class BuildHandler(HandlerClass):
    STREAM_EXPR = {
        "build_success": re.compile("^Successfully built\s+(?P<id>\S+)\n?$")
    }

    def process(self):
        try:
            self.git = git.Git()
            self.docker = context.settings.docker

            with TemporaryFolder() as path:
                self.prepare(path)
                try:
                    self.data.update({"id": self.build(path)})
                except Exception as e:
                    self.data.update({'error': e})

                self.data.update({'build_log': self.build_log})

            if self.data.get('status') and context.settings.options.docker_publish:
                self.push()

            return self.data
        except Exception as e:
            exc = Exception(repr(e))
            exc._tb = traceback.format_exc(e)
            exc.log = getattr(self, 'build_log', [])
            return exc

    def push(self):
        registry = context.settings.options.docker_registry
        use_ssl = context.settings.options.docker_ssl_registry

        tag = self.data['tag'].lstrip("v")
        repo, name = ["".join(list(i)[::-1]).replace('/', '_') for i in self.data['name'].lower()[::-1].split('/', 1)][::-1]

        if not context.settings.options.docker_registry:
            log.warning("PUSHING TO PUBLIC DOCKER REGISTRY.")
        else:
            name = ("%s/%s" % (repo, name)).lower()
            repo = ("%s/%s" % (registry, name)).lower()

            try:
                self.docker.tag(self.data['id'], repo, tag)
            except Exception as e:
                log.error(e)

        log.info(
            "Preparing to push to the registry: %s://%s",
            'https' if use_ssl else 'http', registry if registry else 'PUBLIC'
        )

        response = self.docker.push(
            repo, tag=tag,
            insecure_registry=not use_ssl,
            stream=True
        )

        self.build_log.append('')
        self.build_log.append(
            "Pushing into registry %s://%s" % ('https' if use_ssl else 'http', registry if registry else 'public')
        )

        for line in response:
            chunk = json.loads(line)
            data = chunk.get("status")
            if chunk.get('error'):
                self.data['status'] = False
                details = chunk.get('errorDetail', {}).get('message')
                log.error(details)
                self.build_log.append(details)
            else:
                log.debug(data)

        if registry:
            url = "%s://%s" % ('https' if use_ssl else 'http', registry)
            try:
                log.debug("Trying to fetch image id")
                img_id = filter(lambda x: str(x[0]) == str(tag), requests.get("%s/v1/repositories/%s/tags" % (url, name)).json().items())[0][1]
                log.info('Pushing successful as %s', img_id)
                log.debug("Deleting tag: latest")
                resp = requests.delete("%s/v1/repositories/%s/tags/latest" % (url, name))
                log.debug('%s', resp.json())

                log.debug("Setting latest tag as %s", img_id)
                resp = requests.put(
                    "%s/v1/repositories/%s/tags/latest" % (url, name),
                    '"%s"' % img_id,
                    headers={'Content-Type': 'application/json'}
                )
                log.debug('%s', resp.json())
            except Exception:
                self.build_log.append("ERROR: Can't fetch image id from registry \"%s\"" % url)

    def prepare(self, path):
        url = self.data['repo']
        log.info('Cloning repo "%s" => "%s"', url, path)
        res = self.git.clone(url, path, recursive=True)
        log.debug("Cloning result: %s", res)

        commit_hash = self.data['commit']
        log.info('Checkout commit "%s"', commit_hash)
        self.git.checkout(commit_hash)

        log.info('Updating submodules')
        for sm in git.Repo(path).submodules:
            log.info(' Updating submodule: "%s"', sm)
            sm.update(recursive=True, init=True)
            log.info(' Submodule "%s" updated', sm)

        self.restore_commit_times(path)

        log.info("Preparing complete")

    @staticmethod
    def restore_commit_times(path):

        log.info("Restoring file mtimes for path: %s", path)

        def walk(tree):
            ret = list()
            for i in tree:
                ret.append(i)
                if i.type == 'tree':
                    ret.extend(walk(i))
            return ret

        repo = git.Repo(path)

        def find_mtimes(repo):
            objects = walk(repo.tree())
            t = repo.head.commit
            tt = t.traverse()
            ret = {}
            while objects:
                hashes = set(i.binsha for i in walk(t.tree))
                # iterate over reversed list to be able to remove elements by index
                for n, i in reversed(list(enumerate(objects))):
                    if i.binsha not in hashes:
                        del objects[n]
                    else:
                        if i.path not in ret or t.authored_date < ret[i.path]:
                            ret[i.path] = t.authored_date
                try:
                    t = next(tt)
                except StopIteration:
                    break
            return ret

        for i, mtime in find_mtimes(repo).items():
            fname = os.path.join(path, i)
            log.debug("%s %s", mtime, fname)
            os.utime(fname.encode('utf-8'), (mtime, mtime))

        for sm in repo.submodules:
            sm_internal_path = os.path.join(repo.git_dir, 'modules', sm.name)
            if os.path.exists(sm_internal_path):
                sm_repo = git.Repo(sm_internal_path)
            else:
                sm_repo = git.Repo(sm.path)

            for i, mtime in find_mtimes(sm_repo).items():
                fname = os.path.join(path, sm.path, i)
                log.debug(u"%s %s", mtime, fname)
                os.utime(fname.encode('utf-8'), (mtime, mtime))

    def build(self, path):
        log.debug("Start building...")
        tag = ("%s:%s" % (self.data['name'], self.data['tag'].lstrip("v"))).lower()
        log.debug("Selecting tag: %s", tag)

        self.build_log = []
        log.debug('Building')
        try:
            for line in self.docker.build(path, rm=True, tag=tag):
                chunk = json.loads(line)
                stream = chunk.get("stream", "").rstrip("\n\r")
                if stream:
                    success = self.STREAM_EXPR['build_success'].match(stream)
                    self.build_log.append(stream)
                    if success:
                        self.data['status'] = True
                        return success.groupdict()['id']
                    else:
                        log.info(stream)

                elif chunk.get("error"):
                    err = chunk['error'].strip("\n\r")
                    log.error(err)
                    self.build_log.append(err)
                    log.error(chunk.get('error'))
                    raise StandardError(chunk['error'])
        except Exception as e:
            log.exception(e)

    def resolve_image_id(self, image_id):
        self.docker.images()


BuildHandler.bind("build")



#!/usr/bin/env python
# encoding: utf-8
from functools import wraps
import logging

log = logging.getLogger("handlers")

HANDLERS = []

def register(*urls):
    def deco(cls):
        global HANDLERS

        for url in urls:
            HANDLERS.append((url, cls))
            log.debug('Register URL %r as %r' % (url, cls))

        return cls
    return deco

import handlers


#!/usr/bin/env python
# encoding: utf-8
from collections import namedtuple

from crew.worker import Listener, Context, NODE_UUID, UUID
from pika import PlainCredentials
from crew.worker import context
import logging
import lumper.mailer

def run(args):
    log = logging.getLogger("main")
    try:
        SMTPSettings = namedtuple("SmtpSettings", "host port user password tls sender")
        Listener(
            port=args.amqp_port,
            host=args.amqp_address,
            credentials=PlainCredentials(username=args.amqp_user, password=args.amqp_password) if args.amqp_user else None,
            virtual_host=args.amqp_vhost,
            handlers=context.handlers,
            set_context=Context(
                options=args,
                node_uuid=NODE_UUID,
                uuid=UUID,
                smtp=SMTPSettings(
                    host=args.smtp_host,
                    port=args.smtp_port,
                    user=args.smtp_user,
                    password=args.smtp_password,
                    tls=args.smtp_tls,
                    sender=args.smtp_sender
                )
            ),
        ).loop()
    except Exception as e:
        if logging.getLogger().level < logging.INFO:
            log.exception(e)
        else:
            log.fatal("Exiting by fatal error: %s", e)
    return 0



#!/usr/bin/env python
# encoding: utf-8

from time import time
import tornado.web
import tornado.gen
import hmac
import hashlib
import re
import arrow

from ..json_handler import JSONRequest
from .. import register
from tornado.gen import Future
from tornado.log import app_log as log

REXP={
    "split_refs": re.compile("^refs\/(?P<key>\S+)\/(?P<value>\S+)")
}

@register("/github/webhook/")
@register("/webhook/github")
class GitHubWebHookHandler(JSONRequest):
    @tornado.web.asynchronous
    @tornado.gen.coroutine
    def post(self, *args, **kwargs):
        self.delivery = self.request.headers.get("X-Github-Delivery")
        self.event = self.request.headers.get("X-Github-Event")

        is_valid = self.event and self.delivery and self.verify() if self.settings['args'].github_secret else None

        if is_valid:
            handler = getattr(self, "event_%s" % self.event, lambda: self.response("OK"))
            resp = handler()

            if isinstance(resp, Future):
                ret = yield resp
            else:
                ret = resp

            if not self._finished:
                self.finish(ret)
        else:
            self.send_error(403)
            if not self._finished:
                self.finish()

    def verify(self):
        signature = self.request.headers.get("X-Hub-Signature")
        h = hmac.new(self.settings['args'].github_secret, self.request.body, hashlib.sha1)
        return "sha1=%s" % h.hexdigest() == signature

    def event_ping(self):
        """ Just ping. """
        log.debug("Got PING request: %s", self.json)
        self.send_error(204)

    def event_push(self):
        """ Any Git push to a Repository, including editing tags or branches. Commits via API actions that update
        references are also counted. This is the default event."""
        return self._process_tag()

    def _process_tag(self):
        matcher = REXP['split_refs'].match(self.json.get('ref', ""))
        if matcher:
            refs = matcher.groupdict()
            if refs['key'] == 'tags':
                tag = refs['value']
                commit = self.json.get("head_commit")
                data = {
                    "tag": tag,
                    "repo": self.json['repository']['ssh_url'],
                    "commit": commit['id'],
                    "message": commit['message'],
                    "timestamp": arrow.get(commit['timestamp']).timestamp,
                    "name": self.json['repository']['full_name'],
                    "sender": self.json['sender']['login']
                }

                self.settings['crew'].call("build", data, routing_key="crew.tasks.build.finished", expiration=self.settings.get('timeout', 600))
                self.response(True)
        else:
            self.response(False)

    # def event_commit_comment(self):
    #     """ Any time a Commit is commented on. """
    # def event_delete(self):
    #     """ Any time a Branch or Tag is deleted. """
    # def event_deployment(self):
    #     """ Any time a Repository has a new deployment created from the API. """
    # def event_deployment_status(self):
    #     """ Any time a deployment for a Repository has a status update from the API. """
    # def event_fork(self):
    #     """ Any time a Repository is forked. """
    # def event_gollum(self):
    #     """ Any time a Wiki page is updated. """
    # def event_issue_comment(self):
    #     """ Any time an Issue is commented on. """
    # def event_issues(self):
    #     """ Any time an Issue is assigned, unassigned, labeled, unlabeled, opened, closed, or reopened. """
    # def event_member(self):
    #     """ Any time a User is added as a collaborator to a non-Organization Repository. """
    # def event_membership(self):
    #     """ Any time a User is added or removed from a team. Organization hooks only. """
    # def event_page_build(self):
    #     """ Any time a Pages site is built or results in a failed build. """
    # def event_public(self):
    #     """ Any time a Repository changes from private to public. """
    # def event_pull_request_review_comment(self):
    #     """ Any time a Commit is commented on while inside a Pull Request review (the Files Changed tab). """
    # def event_pull_request(self):
    #     """ Any time a Pull Request is assigned, unassigned, labeled, unlabeled, opened, closed, reopened, or
    #     synchronized (updated due to a (selevent_f):new push in the branch that the pull request is tracking)."""
    # def event_repository(self):
    #     """ Any time a Repository is created. Organization hooks only. """
    # def event_release(self):
    #     """ Any time a Release is published in a Repository. """
    # def event_status(self):
    #     """ Any time a Repository has a status update from the API """
    # def event_team_add(self):
    #     """ Any time a team is added or modified on a Repository. """
    # def event_watch(self):
    #     """ Any time a User watches a Repository. """


@register("/webhook/gitlab")
class CommonWebHookHandler(JSONRequest):

    @tornado.web.asynchronous
    @tornado.gen.coroutine
    def post(self, *args, **kwargs):
        matcher = REXP['split_refs'].match(self.json.get('ref', ""))
        if matcher:
            refs = matcher.groupdict()
            if refs['key'] == 'tags':
                tag = refs['value']
                commit = self.json.get('commits', [])
                commit = commit[0] if commit else {}

                repo_name = self.json['repository']['homepage'].split("/")
                repo_name = "%s/%s" % (repo_name.pop(-2), repo_name.pop())

                data = {
                    "tag": tag,
                    "repo": self.json['repository']['url'],
                    "commit": self.json['checkout_sha'],
                    "message": commit.get('message', '<No message>'),
                    "timestamp": arrow.get(commit.get('timestamp', time())).timestamp,
                    "name": repo_name,
                    "sender": str(self.json['user_id']),
                }

                self.settings['crew'].call("build", data, routing_key="crew.tasks.build.finished", expiration=self.settings.get('timeout', 600))
                self.response(True)
        else:
            self.response(False)



#!/usr/bin/env python
# encoding: utf-8
from crew.worker import Listener, Context, NODE_UUID, UUID
from pika import PlainCredentials
from crew.worker import context
import logging
import docker
import docker.tls
import lumper.worker


def run(args):
    log = logging.getLogger("main")

    if args.docker_tls:
        tls = docker.tls.TLSConfig(client_cert=(args.docker_client_cert, args.docker_client_key),
                                   ca_cert=args.docker_ca_cert, assert_hostname=False)
    else:
        tls = False

    docker_client = docker.Client(base_url=args.docker_url, tls=tls, timeout=300)
    docker_client.verify = args.docker_tls_strict

    try:
        log.info('Testing docker connection: %s', args.docker_url)
        docker_client.info()

        Listener(
            port=args.amqp_port,
            host=args.amqp_address,
            credentials=PlainCredentials(username=args.amqp_user, password=args.amqp_password) if args.amqp_user else None,
            virtual_host=args.amqp_vhost,
            handlers=context.handlers,
            set_context=Context(
                options=args,
                node_uuid=NODE_UUID,
                uuid=UUID,
                heartbeat_counter=0,
                docker=docker_client
            )
        ).loop()
    except Exception as e:
        if logging.getLogger().level < logging.INFO:
            log.exception(e)
        else:
            log.fatal("Exiting by fatal error: %s", e)
    return 0



#!/usr/bin/env python
# encoding: utf-8

from crew.worker import context, Task
from time import time

@Task("heartbeat")
def heartbeat(data):
    context.settings.heartbeat_counter += 1
    return context.settings.uuid, time(), context.settings.heartbeat_counter


#!/usr/bin/env python
# encoding: utf-8
from datetime import datetime
import json
import urllib2
from crew.worker import context, Task
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email import Encoders
import logging
import smtplib


class Attachment(object):
    pass


class FileAttachment(Attachment):
    def __init__(self, data, file_name, content_type="application/octet-stream"):
        self.part = MIMEBase('application', "octet-stream")
        self.part.set_payload(data.encode('utf-8'))
        Encoders.encode_base64(self.part)
        self.part.add_header('Content-Disposition', 'attachment; filename="{0}"'.format(file_name))

    def attach(self, email):
        assert isinstance(email, Email)
        email.add_part(self.part)


class Email(object):
    log = logging.getLogger('emailer.email')

    def __init__(self, sender, recipient, subject, headers={}):
        self.__sent = False
        self.msg = MIMEMultipart()
        self.msg['Subject'] = subject
        self.msg['From'] = sender
        self.msg['To'] = recipient
        self.log.debug("Building message from <%s> to <%s>", sender, recipient)
        for key, value in headers.iteritems():
            self.log.debug('Add header "%s": "%s"', key, value)
            self.msg[key] = value

    def __setitem__(self, key, value):
        self.log.debug('Setting header "%s": "%s"', key, value)
        self.msg[key] = value

    def __getitem__(self, item):
        return self.msg[item]

    def get(self, key, default=None):
        if key in self.msg:
            return self.msg[key]
        else:
            return default

    def add_part(self, part):
        assert isinstance(part, MIMEBase)
        self.msg.attach(part)

    def append(self, part, mimetype='plain'):
        prt = unicode(part)
        self.log.debug('Adding "%s" (length: %s) part to message from <%s> to <%s>', mimetype, len(prt), self.msg['From'], self.msg['To'])
        self.msg.attach(MIMEText(prt, mimetype, _charset='utf-8'))

    def send(self, host='localhost', port=25, tls=False, user=None, password=None):
        self.log.info("Sending message for <%s>.", self.msg['To'])
        self.log.debug("Connecting to SMTP server %s:%d", host, port)
        if not self.__sent:
            connect = smtplib.SMTP(host, port=port)
            try:
                connect.ehlo()
                if tls:
                    self.log.debug("Establishing TLS")
                    connect.starttls()
                    connect.ehlo()
                if user:
                    connect.login(user=user, password=password)
                connect.sendmail(self.msg['From'], self.msg['To'], self.msg.as_string())
                self.__sent = True
                return True
            except Exception as e:
                self.log.exception(e)
                self.log.error("Error: %r", e)
                self.__sent = False
                return False
            finally:
                connect.close()
        else:
            self.log.error("Message already sent")
            return False

log = logging.getLogger('emailer.task')

@Task("build.finished")
def on_build(data):
    if isinstance(data, Exception):
        email = Email(
            sender=context.settings.smtp.sender,
            recipient=context.settings.options.admin_mail,
            subject="Build exception"
        )

        email.append("Error: %r\n\nTraceback: %s\n\n" % (data, getattr(data, '_tb', "No traceback")))

        FileAttachment(
            "\n".join(getattr(data, "log", ['No log',])),
            file_name='build_log.txt',
            content_type='text/plain'
        ).attach(email)

    else:
        if context.settings.options.mail_map:
            recepient = context.settings.options.mail_map.get(data['sender'], context.settings.options.admin_mail)
        else:
            recepient = context.settings.options.admin_mail

        email = Email(
            sender=context.settings.smtp.sender,
            recipient=recepient,
            subject="[%s] <%s> Build %s" % (
                data.get('tag'),
                data.get('name'),
                'successful' if data.get('status') else 'failed'
            )
        )


        email.append(
            "\n".join([
                "Build %s %s" % (data.get('name'), 'successful' if data.get('status') else 'failed'),
                "\n",
                "Sender: %s" % data.get('sender'),
                "Repository: %s" % data.get('repo'),
                "Commit: %s" % data.get('commit'),
                "Commit message: %s" % data.get('message'),
                "Tag: %s" % data.get('tag'),
                "Build timestamp: %s" % data.get('timestamp'),
                "Build date: %s" % datetime.utcfromtimestamp(data['timestamp']) if data.get('timestamp') else None,
                "\n\n"
            ]))

        FileAttachment(
            "\n".join(data.get('build_log')),
            file_name='build_log.txt',
            content_type='text/plain'
        ).attach(email)

        if context.settings.options.build_hooks:
            hook_data = json.dumps(data, sort_keys=False, encoding="utf-8")
            for url in context.settings.options.build_hooks:
                try:
                    log.info('Sending build data to: "%s"', url)
                    req = urllib2.Request(url)
                    req.add_header('Content-Type', 'application/json')
                    response = urllib2.urlopen(req, hook_data, timeout=3)
                    log.info('Build hook response for "%s": HTTP %d', url, response.code)
                except Exception as e:
                    if log.getEffectiveLevel() <= logging.DEBUG:
                        log.exception(e)
                    log.error("Build hook error: %r", e)

    return email.send(
            host=context.settings.smtp.host,
            port=context.settings.smtp.port,
            user=context.settings.smtp.user,
            password=context.settings.smtp.password,
            tls=context.settings.smtp.tls
        )



#!/usr/bin/env python
# encoding: utf-8
from __future__ import absolute_import, print_function
from setuptools import find_packages
import os

# def rpath(*start_paths):
#     packages = set()
#     for start_path in start_paths:
#         if os.path.exists(os.path.join(start_path, '__init__.py')):
#             packages.add(start_path)
#             for cdir, dirs, files in os.walk(start_path):
#                 for d in dirs:
#                     path = os.path.join(cdir, d)
#                     if os.path.exists(os.path.join(path, '__init__.py')):
#                         packages.add(path.replace(os.sep, "."))
#     return list(packages)

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


__version__ = '0.2.24'
__author__ = 'Dmitry Orlov <me@mosquito.su>'


supports = {
    'install_requires': [
        'tornado',
        'arconfig',
        'crew',
        'gitpython',
        'docker-py',
        'arrow'
    ]
}

setup(
    name='lumper',
    version=__version__,
    author=__author__,
    author_email='me@mosquito.su',
    license="MIT",
    description="Containers builder for docker.",
    platforms="all",
    url="http://github.com/mosquito/lumper",
    classifiers=[
        'Environment :: Console',
        'Programming Language :: Python',
    ],
    scripts=['bin/lumper'],
    # package_data={
    #     'lumper.some': ['rc/*'],
    # },
    long_description=open('README.rst').read(),
    packages=find_packages(),
    **supports
)



#!/usr/bin/env python
# encoding: utf-8
from __future__ import division, absolute_import
from ..json_handler import JSONRequest
from .. import register
import tornado.gen
from time import time


@register(r"/api/v1/heartbeat")
class HeartBeat(JSONRequest):
    SUPPORTED_METHODS = ('GET')

    @tornado.gen.coroutine
    def get(self):
        start = time()
        uuid, result, counter = yield self.settings['crew'].call('heartbeat')
        self.response({
            "delta": (result - start) * 1000,
            "uuid": uuid,
            "beats": counter
        })


#!/usr/bin/env python
# encoding: utf-8
import argparse
import logging
import sys


class LogSetterAction(argparse.Action):
    FORMAT = u'[%(asctime)s] %(filename)s:%(lineno)d %(levelname)-6s %(message)s'

    def __init__(self, option_strings, dest, nargs=None, **kwargs):
        if nargs is not None:
            raise ValueError("nargs not allowed")

        logging.basicConfig(format=self.FORMAT, level=logging.INFO)
        super(self.__class__, self).__init__(option_strings, dest, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        self.setter(values)
        setattr(namespace, self.dest, values)

    def setter(self, level='info'):
        root = logging.getLogger()

        level = getattr(logging, level.upper(), logging.INFO)

        fmt = root.handlers[0].formatter
        if root.handlers:
            for handler in root.handlers:
                root.removeHandler(handler)

        handler = logging.StreamHandler(stream=sys.stderr)
        handler.setFormatter(fmt)

        lvl = root.level if not level else level

        root.addHandler(handler)
        root.setLevel(lvl)

        root.info('Logging level is "{0}"'.format(logging._levelNames[lvl]))